#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
web application

author: Tina Wang
last edited: January 2018
"""
import sys
import time
import dbus
import argparse

try:
    import asyncio
except ImportError:
    # Trollius >= 0.3 was renamed
    import trollius as asyncio

from PyQt5 import QtCore
from PyQt5.QtWidgets import (QApplication, QWidget)

from DellLinuxAssistant.portal.common import (MainDialog, BoardDialog, ProductWindow)

from DellLinuxAssistant.telemetry.telemetry_common import (DOMAIN, LOCALEDIR,
                                           setup_logging,
                                           dbus_sync_call_signal_wrapper)
from DellLinuxAssistant.telemetry.dbus_helper import DbusHandler
from DellLinuxAssistant.registration.utils.RegistrationReminder import RegistrationReminder
from gettext import bindtextdomain, textdomain

# SVG file directory
SVGDIR = '/usr/share/pixmaps'
UIDIR = '/usr/share/dell-linux-assistant'


class WorkThread(QtCore.QThread):
    """main thread"""
    main_page_trigger = QtCore.pyqtSignal()
    server_socket = None

    def __int__(self):
        super(WorkThread, self).__init__()

    def run(self):
        """
        main thread run start
        :return:
        """
        time.sleep(3)

        # thread single call App callback
        self.main_page_trigger.emit()

    def __del__(self):
        """
        destructor func
        :return:
        """
        self.wait()


class App(QWidget):

    def __init__(self):
        super().__init__()
        self.main_page = None
        self.work_thread = None
        self.product_page = None

        self.on_board_page = None
        self.dbus_handler = DbusHandler()

        # setup locales
        bindtextdomain(DOMAIN, LOCALEDIR)
        textdomain(DOMAIN)

        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

        self.run()

    def main_page_hide(self):
        """main page single callback, Restore message background-color white"""
        self.product_page_show()
        self.main_page.hide()

    def product_page_show(self):
        """
        product page use pyqt and add web browser
        :return:
        """
        self.product_page = ProductWindow()
        self.product_page.show()

    def run(self):

        usage = '%(prog)s [options]'
        parser = argparse.ArgumentParser(usage=usage)
        parser.add_argument('-d', '--debug', dest='debug', default=False,
                            help='Support debug mode for develop.')
        parser.add_argument('-r', '--reminder', action='store_true', dest='reminder_mode',
                            help='Set application launch as user registration reminder process')
        args = parser.parse_args()

        # set logging init
        if args.debug:
            debug = 'yes'
        else:
            debug = 'no'
        setup_logging(debug=(debug == 'yes'))

        # Verify whether in notification mode
        if args.reminder_mode:
            reg_reminder = RegistrationReminder()
            reg_reminder.notify()

        self.main_page = MainDialog()

        # close the first window then launch the welcome windows
        self.work_thread = WorkThread()
        self.work_thread.main_page_trigger.connect(self.main_page_hide)
        self.work_thread.start()

        try:
            dbus_sync_call_signal_wrapper(self.dbus_handler.backend(),
                                          "deal_message",
                                          {},
                                          "start")
        except dbus.DBusException as msg:
            self.deleteLater()
            self.dbus_handler.dbus_exception_handler(msg)

        self.main_page.show()

    def __del__(self):
        """
        Closes any open backend connections and stops QT threads
        :return:
        """
        self.deleteLater()
        try:
            dbus_sync_call_signal_wrapper(self.dbus_handler.backend(),
                                          "deal_message",
                                          {},
                                          "stop")
        except dbus.DBusException as msg:
            self.dbus_handler.dbus_exception_handler(msg)


if __name__ == "__main__":

    import QtGui
    UIDIR = "/usr/share/pixmaps"

    app = QApplication(sys.argv)
    app.setWindowIcon(QtGui.QIcon(UIDIR+'/dell-linux-assistant.svg'))
    ex = App()
    sys.exit(app.exec_())
