#!/usr/bin/env python3
# coding: utf-8

"""
client web/server web slot and js channel slot

Environment: ubuntu 18.04
Author: dell author
Last edited: January 2018

"""

import asyncio

import os
import json
import time

import dbus
import logging
import webbrowser
import lsb_release
from PyQt5.QtCore import QObject, pyqtSlot, QUrl

from DellLinuxAssistant.telemetry.profile_helper import conf, get_log_event_switch
from DellLinuxAssistant.telemetry.telemetry_common import ObjectThread
from DellLinuxAssistant.registration.utils.BILRegistration import MyDellRegistration
from DellLinuxAssistant.registration.utils.IdmRepository import IdmRepository
from DellLinuxAssistant.registration.utils.WeChatFollowCheckAgent import WeChatFollowCheckAgent
import DellLinuxAssistant.registration.utils.RegistrationHelper as RegistrationHelper

UIDIR = "/usr/share/dell-linux-assistant"

class portalSharedObject(QObject):

    def __init__(self, page=None, ProductWindow=None, DbusHandler=None):
        super().__init__()
        self.page = page
        self.dbus_handler = DbusHandler
        self.product_page = ProductWindow

    @pyqtSlot()
    def sys_info_page(self):
        """
        redirect to system info page
        :return:
        """
        try:
            UIDIR = "/usr/share/dell-linux-assistant/portal/html"
            page_url = "file://"+os.path.join(UIDIR, 'systemInfo.html')
            self.page.load(QUrl(page_url))

        except Exception as e:
            logging.error("[Opening System Information Page]ERR:{}".format(str(e)))

    @pyqtSlot()
    def user_reg_page(self):
        """
        redirect to user registration page
        :return:
        """
        country = RegistrationHelper.get_os_country()
        language = RegistrationHelper.get_os_language()
        self.product_page.switch_broswer('production',
                                         country,
                                         language,
                                         False)
        logging.info("[client registration browser msg]INFO:{}".format(str('Registration Load!')))

    @pyqtSlot(result=bool)
    def get_registration_event(self):
        """
        get registration event
        :return:
        """
        self.BILRegistration = MyDellRegistration()
        return self.BILRegistration.GetRegistrationEvent()

    @pyqtSlot(result=str)
    def get_license(self):
        """
        get all thrid party licenses
        :return: license
        """
        license_dir = "/usr/share/dell-linux-assistant/LICENSES"
        license_list = os.listdir(license_dir)
        content = ""
        for license in license_list:
            with open(os.path.join(license_dir, license), 'r') as license_file:
                content += license_file.read()

            # add empty lines between each license
            content += "\n\n\n\n\n"
        return content.replace("\n", "<br>")

    @pyqtSlot(result=str)
    def get_privacy(self):
        """
        get privacy notice
        :return: privacy content
        """
        base_dir = "/usr/share/dell-linux-assistant"
        with open(os.path.join(base_dir, 'TELEMETRY_PRIVACY'), 'r') as privacy_file:
            content = privacy_file.read()
            return content.replace("\n", "<br>")

    @pyqtSlot(str, str)
    def logMetricEvent(self, tag, value):
        """
        server web log event
        :param tag: name
        :param value: value
        :return:
        """
        self.dbus_handler.logMetricEvent(tag, value)


class SystemInfoSharedObject(QObject):
    def __init__(self, page=None, DbusHandler=None):
        super().__init__()
        self.page = page
        self.system_profile = ()

        self.dbus_handler = DbusHandler
        self.dell_support_url = conf['Dell_Support_Url']
        self.os_support_url = conf['OS_Support_Url']

    @pyqtSlot(result=str)
    def get_os_support_profile(self):
        """
        the method is to call dbus service to provide those system's HW data
        :return: profile info json format
        (serialnumber, bios_version, kernel, product_name,
        cpu, mem, graphic, disk, ethernet, wireless, system_version,
        osLanguage, oobeDate)
        """

        try:

            if not self.system_profile:
                self.system_profile = self.dbus_handler.backend().get_HW_info()

            os_profile_dict = {
                'Kernel Version': self.system_profile[2],
                'CPU': self.system_profile[4],
                'Memory': self.system_profile[5]
            }

            os_profile_dict['Disks'] = self.system_profile[7].replace('\n', '<br>')
            os_profile_dict['Graphics'] = self.system_profile[6].replace('\n', '<br>')

            Ethernet = self.system_profile[8]
            Wireless = self.system_profile[9]
            if Ethernet and Ethernet != 'unknow':
                os_profile_dict['Ethernet'] = Ethernet

            if Wireless and Wireless != 'unknow':
                os_profile_dict['Wireless'] = Wireless

            return json.dumps(os_profile_dict)

        except dbus.DBusException as msg:
            self.dbus_handler.dbus_exception_handler(msg)
            return ""

    @pyqtSlot(result=str)
    def get_dell_support_profile(self):
        """
        Get dell system profile
        :return: profile info json format
        """
        try:
            if not self.system_profile:
                self.system_profile = self.dbus_handler.backend().get_HW_info()

            profile = {
                'Service Tag': self.system_profile[0],
                'BIOS Version': self.system_profile[1],
            }

            return json.dumps({
                'dell_profile': profile,
                'Model': self.system_profile[3]
            })

        except dbus.DBusException as msg:
            self.dbus_handler.dbus_exception_handler(msg)
            return ""

    @pyqtSlot()
    def os_support_btn(self):
        """
        os support web click event
        :return:
        """
        try:
            url = os.path.join(self.os_support_url,
                               lsb_release.get_distro_information()['ID'].lower())
            webbrowser.open(url)
        except Exception as e:
            logging.error("[os_support button]ERR:{}".format(str(e)))

    @pyqtSlot()
    def dell_support_btn(self):
        """
        dell support web click event
        :return:
        """
        try:
            if not self.system_profile:
                self.system_profile = self.dbus_handler.backend().get_HW_info()

            service_tag = self.system_profile[0]
            webbrowser.open(os.path.join(self.dell_support_url, service_tag))
        except Exception as e:
            logging.error("[dell_support button]ERR:{}".format(str(e)))

    @pyqtSlot(bool)
    def log_usage_event(self, status):
        """
        Enable usage data to be sent to Dell
        :param status: True or False
        :return:
        """
        logging.info("Enable usage data to be sent to Dell")
        self.dbus_handler.LogUsageEvent(True if status else False)

    @pyqtSlot(result=bool)
    def get_log_usage_event(self):
        """
        usage data to be sent to Dell
        """
        return get_log_event_switch()

    @pyqtSlot()
    def back_to_homepage(self):
        """
        redirect to homepage
        :return:
        """
        try:
            # set page url
            UIDIR = "/usr/share/dell-linux-assistant/portal/html"
            page_url = "file://"+os.path.join(UIDIR, 'portal.html')
            self.page.load(QUrl(page_url))
        except Exception as e:
            logging.error("[Opening Homepage]ERR:{}".format(str(e)))

class RegistrationClientSharedObject(QObject):
    """
    Registration client within app
    """
    def __init__(self, ProductWindow=None, DbusHandler=None):
        super().__init__()
        self.product_page = ProductWindow
        self.dbus_handler = DbusHandler


    @pyqtSlot(str, str, str, bool)
    def registration_massage(self, selectedEnvironment, selectedCountry, selectedLanguage, selectedRetailOption):
        """
        web event massage
        :param msg: event name
        :return:
        """
        try:
            if selectedEnvironment not in ['Dev', 'Qa', 'Staging', 'Production']:
                self.dbus_handler.dbus_massage('environment error!')
                return

            system_country = json.loads(self.get_system_country(True))
            if not system_country.get(selectedCountry):
                self.dbus_handler.dbus_massage('country error!')
                return

            system_language = json.loads(self.get_system_language(True))
            if not system_language.get(selectedLanguage):
                self.dbus_handler.dbus_massage('language error!')
                return

            if selectedRetailOption not in [True, False]:
                self.dbus_handler.dbus_massage('retail error!')
                return

            self.product_page.switch_broswer(selectedEnvironment,
                                     system_country.get(selectedCountry),
                                     system_language.get(selectedLanguage),
                                     selectedRetailOption)
            logging.info("[client registration browser socket msg]INFO:{}".format(str('Registration Load!')))

        except Exception as err:
            logging.error("[client registration browser socket msg]ERROR:{}".format(str(err)))

    @ObjectThread()
    @pyqtSlot(result=str)
    def get_system_language(self, isValue=False):
        """
        get mockUI language, set server page setting
        :return:
        """
        try:
            with open(UIDIR + '/registration/jsons/LanguageList.json', 'r') as json_file:
                content = json_file.read()

                if content.startswith(u'\ufeff'):
                    content = content.encode('utf8')[3:].decode('utf8')

                language = json.loads(content)
                if isValue:
                    languages = {item['Name']:item['LanguageCode'] for item in language}
                else:
                    languages = [item['Name'] for item in language]

                json_file.close()

                return json.dumps(languages)
        except Exception as err:
            logging.error('[HTML] get system language: %s' % str(err))
            return ""

    @ObjectThread()
    @pyqtSlot(result=str)
    def get_system_country(self, isValue=False):
        """
        get mockUI country, for server page test setting
        :return:
        """
        try:
            with open(UIDIR + '/registration/jsons/CountryList.json', 'r') as json_file:
                content = json_file.read()

                if content.startswith(u'\ufeff'):
                    content = content.encode('utf8')[3:].decode('utf8')

                country = json.loads(content)
                if isValue:
                    countrys = {item['Name']:item['TwoLetterIsoCode'] for item in country}
                else:
                    countrys = [item['Name'] for item in country]

                json_file.close()

                return json.dumps(countrys)
        except Exception as err:
            logging.error('[HTML] get system country: %s' % str(err))
            return ""

class RegistrationSharedObject(QObject):
    """
    Registration server side js shared object
    """
    def __init__(self,
                 ProductWindow=None,
                 clientAppVersion=None,
                 DbusHandler=None):
        super().__init__()
        self.product_page=ProductWindow
        self.idm = IdmRepository()
        self.dbus_handler = DbusHandler
        self.BILRegistration = MyDellRegistration()
        self.wechatAgent = WeChatFollowCheckAgent()
        self.clientAppVersion = clientAppVersion

        self.IsTestMode = True

    @pyqtSlot(str, str)
    def logMetricEvent(self, tag, value):
        """
        server web log event
        :param tag: name
        :param value: value
        :return:
        """
        self.dbus_handler.logMetricEvent(tag, value)

    def isNetworkAvailable(self):
        """
        check network async
        :return:
        """
        networkAvailable = False
        Network = self.dbus_handler.backend().get_net_work()

        if Network == 'ABLED':
            networkAvailable = True
        else:
            self.dbus_handler.logEvent("Offline message displayed")

        if not networkAvailable:
            self.dbus_handler.dbus_massage("You are currently offline")

        return networkAvailable

    @pyqtSlot(result=bool)
    def checkInternetAccess(self):
        """
        check network available
        :return: bool
        """
        return self.isNetworkAvailable()


    def CreateAccount(self, jsonData):
        """
        create account
        :param jsonData: user info
        :return: bool
        """
        if not jsonData:
            self.dbus_handler.logEvent("Idm create account called with no data")
            return False

        return self.idm.TryCreateAccount(jsonData)

    @pyqtSlot(str)
    def createAccount(self, jsonData):
        """
        create account
        :param bilFormData: json data
        :return: bool
        """
        res= self.CreateAccount(jsonData)
        return res

    def DoRegistration(self, formData, email, jsonFormData, serviceTag):

        return self.BILRegistration.DoRegistration(formData, email, jsonFormData, serviceTag)

    @pyqtSlot(str, str, str, str, result=bool)
    def doRegistration(self, bilFormData, email, telemetryJsonObject, st):
        """
        registration event
        :param bilFormData: form data
        :param email: email
        :param telemetryJsonObject: json object
        :param st: tag
        :return: bool
        """
        status = self.DoRegistration(bilFormData, email, telemetryJsonObject, st)
        self.BILRegistration.SetRegistrationEvent(status)
        if not status:
            self.dbus_handler.dbus_massage('Oops!\nRegistration failed. There has been an error while registering. We apologize for any inconvenience. Please try again later.')

        return status

    # @ObjectThread()
    @pyqtSlot(result=str)
    def getClientAppVersion(self):
        """
        get version
        :return: client app version: string
        """
        return self.clientAppVersion

    # @ObjectThread()
    @pyqtSlot(result=bool)
    def isTestMode(self):
        """
        get test model
        :return: bool
        """
        return self.IsTestMode

    def IsWeChatFollowed(self, telemetryType):

        wechatStatus = self.wechatAgent.CheckAsync(telemetryType)
        return wechatStatus

    @pyqtSlot(str, str, str, result=bool)
    def checkWeChatFollowStatus(self, maxTime, intervalTime, telemetryType):
        """
        check wechat status
        :param maxTime: max request time, no use
        :param intervalTime:  intervalTime, no use
        :param telemetryType:
        :return:
        """
        status = self.IsWeChatFollowed(telemetryType)
        return status

    @pyqtSlot()
    def closeWebview(self):
        """
        close web view, return back
        :return:
        """
        try:
            # not back to homepage when in hidden pre-load page
            if self.product_page._load_start:
                self.product_page.main_browser()
        except Exception as err:
            logging.info("[web browser close msg]ERROR:{}".format(str(err)))


    @pyqtSlot()
    def documentReady(self):
        """
        document version
        :return:
        """
        try:
            self.product_page.GetWebAppVersion()
        except Exception as err:
            logging.info("[web browser documentReady msg]ERROR:{}".format(str(err)))



if __name__ == "__main__":

    # shareobj = Html("2.10")
    # account = shareobj.createAccount("FNAM=dfd&LNAM=dgd&EMAL=waffgfg%40123.com&ADD1=ggh&ADD2=sss&ZIPC=26855&CITY=CABINS&"
    #                   "STAT=WV&PUSE=Personal&DADV=1&NEMA=1&OFBN=XE26NNAR&CTRY=US&RLNG=en_us"
    #                   "&OSYS=Microsoft%20Windows%2010%20Pro&OSVR=10.1.16299&MODL"
    #                   "=Precision%2055660&WDAT=2018-04-18%2017:45:30&WSID=960201&SRCE=FusionReg-9.9.9.99")
    # print("request status = ", account)

    clientobj = RegistrationClientSharedObject()
    dict = clientobj.get_system_language()
    language = json.loads(dict)
    country = clientobj.get_system_country()
    print(language, json.loads(country))
