#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""

This module defines:

    + All file & directory related operation functions.

.. module::DellLinuxAssistant.telemetry.file_helper

    :synopsis: File & Directory related operation.

------

"""

import glob
import logging
import json
import os
import shutil
import threading


def serialize(obj):
    """
    Serialize Event object into JSON format data.

    :return: JSON format event data.
    """
    from DellLinuxAssistant.telemetry.event import Event
    # verify whether obj is the instance of Event
    if isinstance(obj, Event):
        # fields required by Event constructor, no other state
        return {"ObjectType": "DellLinuxAssistant.telemetry.Event",
                "EventTelemetryType": obj.EventTelemetryType,
                "EventType": obj.EventTag,
                "EventValue": obj.EventValue,
                "EventCreatedUtc": str(obj.EventCreatedUtc),
                "Event_UUID": str(obj.Event_UUID)}
    else:
        raise TypeError(f"Object type {obj.__class__.__name__} is not serializable using this encoder")


def deserialize(obj):
    """
    Deserialize source object into Event object.

    :return: Event data object.
    """
    from DellLinuxAssistant.telemetry.event import Event
    if obj["ObjectType"] == "DellLinuxAssistant.telemetry.Event":
        return Event(obj["EventTelemetryType"], obj["EventType"], obj["EventValue"], obj["EventCreatedUtc"], obj["Event_UUID"])
    else:
        raise TypeError("Provided object type is not deserializable using this decoder")


class FileHelper():
    """
    This class integrated all file & directory related functions.
    """
    def __init__(self):
        self.lock = threading.Lock()

    def SaveData(self, data, path):
        """
        Save the device profile information into settings file

        :param data: Event data.
        :type data: Event
        :param path: Target path for data storing.
        :type path: string
        :return: Result status.
        :rtype: bool
        """
        self.lock.acquire()
        if not os.path.exists(path):
            os.system(r'touch %s' % path)
            logging.debug('create new file:{}'.format(path))
        try:
            with open(path, 'w') as output:
                json.dump(data, output, default=serialize)
                logging.debug("dump file into {}".format(path))
            return True
        except Exception as e:
            logging.error("Exception %s" % str(e))
            return False
        finally:
            self.lock.release()

    def GetDataFromFile(self,path, profile_flag=None):
        """
        Read the device profile information from settings file

        :param path: Source path of data file.
        :param profile_flag: Wether the default output is formatted deserialize otherwise
        :type path: string
        :return: Event data
        :rtype: Event
        """
        self.lock.acquire()
        data = None
        if os.path.exists(path):
            with open(path, 'r') as input:
                if profile_flag:
                    data = json.load(input)
                else:
                    data = json.load(input, object_hook=deserialize)
        else:
            shutil.rmtree(path, ignore_errors=True)
        self.lock.release()
        return data

    def DeleteFiles(self, deletionlist):
        """
        Delete files for cleanup

        :param deletionlist: A filename list for deletion.
        :type deletionlist: list
        """
        self.lock.acquire()
        for fileToDelete in deletionlist:
            try:
                if os.path.exists(fileToDelete):
                    os.remove(fileToDelete)
                    # update the cleanup file list
                    deletionlist.remove(fileToDelete)
            except Exception as e:
                logging.error("delete the file failed, error is {}".format(str(e)))
        self.lock.release()

    def DeleteEmptyFolders(self, filePath):
        """
        Delete empty folders

        :param filepath: Target filepath
        :type filepath: string
        """
        self.lock.acquire()
        if os.path.exists(filePath):
            # get all directories in path
            dayDirectories = []
            for _ in os.scandir(filePath):
                if _.is_dir():
                    # get all the "yyyymmdd" timestamp folers ex:1526031684
                    dayDirectories.append(os.path.abspath(_.path))
            # count the file numbers of those directories
            for dayDirectory in dayDirectories:
                if len(os.listdir(dayDirectory)) == 0:
                    try:
                        os.rmdir(dayDirectory)
                    except Exception as e:
                        logging.error("delete the empty folder failed, the error message is {}".format(str(e)))
        self.lock.release()

    def GetNextEventFileDetails(self, size, folderPath):
        """
        Get the next events to be sent in a batch

        :param size: Count of files to process.
        :param folderPath: Root path of event files
        """
        currentCount = 0
        nextEvent = list()
        isCompleted = False
        if os.path.exists(folderPath):
            # get all directories in path
            dayDirectories = []
            for _ in os.scandir(folderPath):
                if _.is_dir():
                    # get all the "yyyymmdd" timestamp folers ex:1526031684
                    dayDirectories.append(os.path.basename(_.path))
            dayDirectories.sort(key=lambda x: float(x))
            # print("dayDirectories:",dayDirectories)
            for DirectoryInfo in dayDirectories:
                # DirectoryInfo will sort from small to big
                if not isCompleted:
                    # print("DirectoryInfo:",DirectoryInfo)
                    # print(os.path.join(DirectoryInfo,'*.txt'))
                    files = glob.glob(os.path.join(folderPath, DirectoryInfo, '*.txt'))
                    # print("folderPath:",folderPath)
                    # print("files:",files)
                    files.sort(key=os.path.getmtime)
                    # print("files:", files)
                    for file in files:
                        # print("file:", file)
                        # print(currentCount)
                        if currentCount < size:
                            nextEvent.append(os.path.join(folderPath, DirectoryInfo, file))
                            # print("nextEvent:",nextEvent)
                            currentCount += 1
                            # print(currentCount)
                        else:
                            isCompleted = True
                            break
                else:
                    break
            return nextEvent

    def DeleteOldestFile(self, path, count=1):
        """
        Delete the oldest event file to save the new one

        :param path: File path of events.
        :param count: Total count of files to be deleted.
        :type path: string
        :type count: int
        """
        # self.lock.acquire()
        oldestEventFile = self.GetNextEventFileDetails(count, path)
        logging.debug("[DeleteOldestFile]the delete file path is {}".format(oldestEventFile))
        if oldestEventFile:
            self.DeleteFiles(oldestEventFile)
            # self.lock.release()

    def UpdateEventFile(self, d_data, path):
        """
        update the device profile information in settings file
        :param d_data: update dict data
        :param path: file path
        :return: True: successd False: fail
        """
        self.lock.acquire()
        if not isinstance(d_data, dict):
            logging.debug("[UpdatetEventFile] d_data type error: {}".format(str(d_data)))
            return False

        if not os.path.exists(path):
            os.system(r'touch %s' % path)
            logging.debug('create new file:{}'.format(path))
            original_data = {}
        else:
            with open(path, 'r') as output:
                original_data = json.load(output)
        try:
            dict_merged = dict(original_data, **d_data)
            with open(path, 'w') as info_input:
                json.dump(dict_merged, info_input)
                logging.debug("[UpdatetEventFile] update file into {}".format(path))
        except Exception as e:
            logging.error("[UpdatetEventFile] [ERROR] %s" % str(e))
            return False
        finally:
            self.lock.release()
            return True


if __name__ == '__main__':
    # FileHelper().DeleteOldestFile("/var/cache/dell-telemetry")
    from DellLinuxAssistant.telemetry.event import Event
    import configparser
    from DellLinuxAssistant.telemetry.telemetry_common import INI_FILE
    config = configparser.ConfigParser()
    config.read(INI_FILE)
    configure = config['TEST']

    event = Event("OTP")
    filepath = event.QueueEvent(configure['PlatinumFolderPath'])
    print(filepath)
    # filepath = "/tmp/test/TestDellLinuxAssistant/ApiKey/1532390400/47822.891696-0f6b57874ce6436fa9d68d6c9184a88a.txt"
    load_event = FileHelper().GetDataFromFile(filepath)
    attrs = vars(load_event)
    print('\n'.join("%s: %s" % _ for _ in attrs.items()))

    # myprofile1 = Profile(os.path.join(configure['PlatinumFolderPath'], configure['Platinum_Setting_Filename']))
    # # s = json.dumps(myprofile1, default=serialize)
    # # print(s)
    # # myprofile2 = json.loads(s, object_hook=deserialize)
    # # print(myprofile2)
    # attrs = vars(myprofile1)
    # print('\n'.join("%s: %s" % _ for _ in attrs.items()))
