#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""

This module defines:

    + Message sending related functions.

.. module::DellLinuxAssistant.telemetry.message_helper

    :synopsis: All message sending related operations.

------

"""
import json
import ctypes
import logging
import os
import time
import threading

from DellLinuxAssistant.telemetry.event import Event
from DellLinuxAssistant.telemetry.file_helper import FileHelper
from DellLinuxAssistant.telemetry.profile import Profile
from DellLinuxAssistant.telemetry.telemetry_common import is_connected, tasklet, check_version


class MessageHelper():
    """
    This is a message helper class that call cpp lib to send message/event.
    """

    def __init__(self, configuration):
        self.lock = threading.Lock()
        self.lib = ctypes.cdll.LoadLibrary("/usr/lib/x86_64-linux-gnu/dell-linux-assistant/dell_linux_assistant.so")
        self.conf = configuration

    def SendMessage(self, message):
        """
        Call C++ lib to send message.

        :param message: Message to be sent.
        :type message: Message
        :return: Return code for send result.
        :raises: RuntimeError
        """
        # verify wether message is a correct data object of Message.
        if isinstance(message, Message):
            # parse message data
            SN = message.profile.SerialNumber.encode()
            logging.debug("SN:{},type:{}".format(SN.decode(), type(SN)))
            ApiUrl = self.conf['ApiUrl'].encode()
            logging.debug("ApiUrl:{},type:{}".format(ApiUrl.decode(), type(ApiUrl)))
            GeoId = message.profile.GeoId.encode()
            logging.debug("GeoId:{},type:{}".format(GeoId.decode(), type(GeoId)))
            AppVersion = check_version('dell-linux-assistant').encode()
            logging.debug("AppVersion:{},type:{}".format(AppVersion.decode(), type(AppVersion)))
            DAID = message.profile.DeviceApplicationId.encode()
            logging.debug("DAID:{},type:{}".format(DAID.decode(), type(DAID)))
            MF = message.profile.Manufacturer.encode()
            logging.debug("MF:{},type:{}".format(MF.decode(), type(MF)))
            Model = message.profile.Model.encode()
            logging.debug("Model:{},type:{}".format(Model.decode(), type(Model)))
            OCD = message.profile.OobeDate.encode()
            logging.debug("OCD:{},type:{}".format(OCD.decode(), type(OCD)))
            OSSku = message.profile.OSSKU.encode()
            logging.debug("OSSku:{},type:{}".format(OSSku.decode(), type(OSSku)))
            OSVersion = message.profile.OsVersion.encode()
            logging.debug("OSVersion:{},type:{}".format(OSVersion.decode(), type(OSVersion)))
            LC = message.profile.OsLanguage.encode()
            logging.debug("LC:{},type:{}".format(LC.decode(), type(LC)))
            TS = str(message.event.EventCreatedUtc).encode()
            logging.debug("TS:{},type:{}".format(TS.decode(), type(TS)))
            ET = message.event.EventTag.encode()
            logging.debug("[SendMessage]ET:{}".format(ET.decode()))
            EV = message.event.EventValue.encode()
            logging.debug("[SendMessage]EV:{}".format(EV.decode()))
            EUID = message.event.Event_UUID.encode()
            logging.debug("[SendMessage]EUID:{}".format(EUID.decode()))
            requestAction = message.get_request_action()
            proxyipaddr = self.conf['proxyipaddr'].encode()
            logging.debug("proxyipaddr:{},type:{}".format(proxyipaddr.decode(), type(proxyipaddr)))
            proxyport = self.conf['proxyport'].encode()
            logging.debug("proxyport:{},type:{}".format(proxyport.decode(), type(proxyport)))
            # sent message to server successfully
            try:
                return_code = self.lib.SendMessage(SN, ApiUrl, GeoId, AppVersion,
                                                   DAID, MF, Model, OCD, OSSku,
                                                   OSVersion, LC, TS, ET, EV, EUID,
                                                   requestAction, proxyipaddr, proxyport)
            except Exception as e:
                logging.error("[SendMessage]ERROR:{}".format(str(e)))
            logging.debug('[SendMessage]:return code is {}'.format(return_code))
            return return_code
        else:
            raise RuntimeError('the data format is not right!')

    # @tasklet(timeout=15, maxretry=1)
    def SendEvent(self, filepath, debug=None, profile=None, Decryption=None):
        """
        Get event date from specific filepath and send at the first time

        :param filepath: Path of event files
        :param profile: Profile detail
        """
        messagesToSend = list()
        sentEventFileDetails = list()
        folderPath = self.conf['PlatinumFolderPath']
        # check profile, if no profile assigned, then create a new one.
        if not profile:
            profile = Profile(os.path.join(self.conf['PlatinumFolderPath'], self.conf['Platinum_Setting_Filename']))
        else:
            profile = profile
        event = FileHelper().GetDataFromFile(filepath)
        logging.info("[SendEvent]currentEvent:{}".format(filepath))
        if event:
            # merge profile into message class data
            message = Message(profile, event)
            # back up for send failed
            messagesToSend.append(message)
            sentEventFileDetails.append(filepath)
            try:
                # check return code in response
                response = self.SendMessage(message)
                if response == 0:
                    # clean up message
                    FileHelper().DeleteFiles(sentEventFileDetails)
                    FileHelper().DeleteEmptyFolders(folderPath)
                elif response == -1:
                    # send message data is broken
                    logging.error("[SendEvent]send message data itself is broken")
                else:
                    sentEventFileDetails.clear()
                    time.sleep(6)
            except Exception as e:
                logging.error("[SendEvent]call sendmessage lib failed error:{}".format(str(e)))
                # if the network is offline
                if not is_connected(self.conf['ApiUrl'].split("/")[2]):
                    sentEventFileDetails.clear()
                else:
                    # try to send again
                    time.sleep(1)
                    try:
                        self.SendMessage(messagesToSend.pop())
                    except Exception as e:
                        logging.error("[SendEvent]re-sent message failed error:{}".format(str(e)))
            finally:
                if len(sentEventFileDetails) > 0:
                    # Issue files cleanup
                    logging.debug("[SentEvent]start to delete files")
                    FileHelper().DeleteFiles(sentEventFileDetails)
                    FileHelper().DeleteEmptyFolders(folderPath)

    @tasklet(timeout=30, maxretry=1)
    def SendBatchEvents(self, batchSize=10, Decryption=None):
        """
        Send messages in batch to event hub

        :param batchSize: Size of batch, default value is 10
        """
        self.lock.acquire()
        try:
            # Build File Paths
            logging.debug("[SendBatchEvents]running SendBatchEvents function ...")
            folderPath = self.conf['PlatinumFolderPath']
            if os.path.exists(folderPath):
                if len(FileHelper().GetNextEventFileDetails(1, folderPath)) > 0:
                    profile = Profile(os.path.join(self.conf['PlatinumFolderPath'], self.conf['Platinum_Setting_Filename']))
                    while True:
                        # sent target messages
                        if self._IsMessagesAvailableToSend(folderPath):
                            # get next event details for sending
                            nextEventFileDetails = FileHelper().GetNextEventFileDetails(batchSize, folderPath)
                            if nextEventFileDetails:
                                for eventFile in nextEventFileDetails:
                                    self.SendEvent(eventFile, profile)
                        else:
                            break
        except Exception as e:
            logging.error("[SendBatchEvents]Exception Error: {0}".format(e))
        finally:
            self.lock.release()

    def _IsMessagesAvailableToSend(self, folderPath):
        '''check if there is any messages unsent for long time'''
        result = False
        if os.path.exists(folderPath):
            # get all directories in path
            dayDirectories = []
            for _ in os.scandir(folderPath):
                if _.is_dir():
                    # get all the "MMddyyyy" folers ex:1526031684.0
                    dayDirectories.append(os.path.basename(_.path))
            if dayDirectories:
                result = True
        return result

    def GenerateSignature(self, apiKey, nounce, timestamp):
        """
        get sign
        :param nounce: nounce number
        :param timestamp:  unix timestamp
        :return:
        """
        try:
            self.lib.GetWechatSignature.restype = ctypes.c_char_p
            return_code = self.lib.GetWechatSignature(apiKey.encode("utf-8"),
                                                      str(timestamp).encode('u8'), nounce.encode('u8'))
            logging.debug('[GetWechatSignature]:return code is {}'.format(return_code))

            res = json.loads(return_code)
            code = res.get('code')
            if code:
                return res.get('msg')

        except Exception as e:
            logging.error("[GetWechatSignature]ERROR:{}".format(str(e)))

        return None

    def getIdentityContent(self, parameterValue, IDMHashKey):
        """
        :param parameterValue: url value
        :param IDMHashKey: hashKey
        :return:
        """
        try:
            self.lib.GetIdentityContent.restype = ctypes.c_char_p
            return_code = self.lib.GetIdentityContent(parameterValue.encode("utf-8"),
                                                      IDMHashKey.encode('u8'))
            logging.debug('[GetIdentityContent]:return code is {}'.format(return_code))

            res = json.loads(return_code)
            code = res.get('code')
            if code:
                return res.get('msg')

        except Exception as e:
            logging.error("[GetIdentityContent]ERROR:{}".format(str(e)))

        return None

    def getAuthInfo(self, is_product):
        """
        Method to get hash
        :param authkey: key
        :return: hash
        """
        try:
            self.lib.GetAuthInfo.restype = ctypes.c_char_p
            return_code = self.lib.GetAuthInfo(is_product)
            logging.debug('[GetAuthInfo]:return code is {}'.format(return_code))

            res = json.loads(return_code)
            code = res.get('code')
            if code:
                return res.get('msg')

        except Exception as e:
            logging.error("[GetAuthInfo]ERROR:{}".format(str(e)))

        return None

    def getSumInput(self, checkSumInput):
        """
        Method to get hash.
        :param checkSumInput: The input checkSumInput
        :return: hash
        """
        try:
            self.lib.GetSumInput.restype = ctypes.c_char_p
            return_code = self.lib.GetSumInput(checkSumInput.encode('u8'))
            logging.debug('[GetSumInput]:return code is {}'.format(return_code))

            res = json.loads(return_code)
            code = res.get('code')
            if code:
                return res.get('msg')

        except Exception as e:
            logging.error("[GetSumInput]ERROR:{}".format(str(e)))

        return None

    def GetSystemProfile(self):
        """
        get system profile info
        :return:
        """

        try:
            self.lib.GetSystemProfile.restype = ctypes.c_char_p
            return_code = self.lib.GetSystemProfile()
            logging.debug('[GetSystemProfile]:return code is {}'.format(return_code))
            res = json.loads(return_code)
            code = res.get('code')
            if code:
                msg = res.get('msg')
                return json.dumps(msg)

        except Exception as e:
            logging.error("[GetSystemProfile]ERROR:{}".format(str(e)))

        return None

    def VerifySSLFingerprint(self, environment_):
        """
        0 if failed, 1 if pass
        :return:
        """

        try:
            code = self.lib.VerifySSLFingerprint(environment_.encode('u8'))
            logging.debug('[VerifySSLFingerprint]:return code is {}'.format(code))

            return code

        except Exception as e:
            logging.error("[VerifySSLFinger]ERROR:{}".format(str(e)))

        return 0

    def GetRegistrationUrl(self, url_name):
        """
        url
        :param url_name: as DEV_BASE_URL
        :return:
        """
        try:
            self.lib.GetRegistrationUrl.restype = ctypes.c_char_p
            return_code = self.lib.GetRegistrationUrl(url_name.encode('u8'))
            logging.debug('[GetRegistrationUrl]:return code is {}'.format(return_code))

            res = json.loads(return_code)
            code = res.get('code')
            if code:
                return res.get('msg')

        except Exception as e:
            logging.error("[GetRegistrationUrl]ERROR:{}".format(str(e)))

        return None


class Message(object):
    """
    It is a message class which will collect all the needed data into one frame to send
    """
    def __init__(self, profile, event):
        self.profile = None
        self.event = None
        if isinstance(profile, Profile):
            self.profile = profile
        else:
            raise RuntimeError("the input value is not a Profile class!")
        if isinstance(event, Event):
            self.event = event
        else:
            raise RuntimeError("the input value is not a Event class!")

    def get_request_action(self):
        """
        Transfer the event type into numbers which is needed by sent message lib API

        :return: Action number
        :rtype: int
        :raises: RuntimeError 
        """
        action = self.event.EventTelemetryType
        if action == 'HB':
            return 1
        elif action == 'OTP':
            return 2
        elif action == 'LogEvent':
            return 3
        elif action == 'LogMetricEvent':
            return 4
        elif action == 'LogComplexEvent':
            return 5
        elif action == 'LogFile':
            return 6
        else:
            raise RuntimeError("the request action is not valid!")
