#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""

This module defines:

    * Profile class that contains HW & SW related data.

.. module::DellLinuxAssistant.telemetry.profile

    :synopsis: Profile class contains HW & SW related data

------

"""

import json
import logging
import lsb_release
import os

from DellLinuxAssistant.telemetry import profile_helper
from DellLinuxAssistant.telemetry.telemetry_common import read_sys_data


class Profile(object):
    """
    This class is only to define the HW and SW related data which are needed by server's https header
    """
    def __init__(self, path):
        # profile configuration
        self.SerialNumber = self._get_SN()
        self.ServiceTag = self.SerialNumber
        self.Manufacturer = self._get_Manufacturer()
        self.Model = self._get_Model()
        self.OobeDate = self._get_oobe_date(path)
        self.OSSKU = self._get_SKU()
        self.GeoId = self._get_geoid()
        self.OsVersion = self._get_os_version()
        self.OsLanguage = self._get_os_lanuage()
        self.DeviceApplicationId = self._get_UUID()

    def _get_SN(self):
        """
        SerialNumber or ServiceTag needs to be got under root privillege

        :return: Profile SN info
        :rtype: string
        """
        SN = None
        try:
            SN = profile_helper.get_SN()
        except Exception as e:
            logging.error("[_get_SN]error is {}".format(str(e)))
        finally:
            return SN

    def _get_UUID(self):
        """
        SerialNumber or ServiceTag needs to be got under root privillege

        :return: Profile UUID info
        :rtype: string
        """
        UUID = None
        try:
            UUID = profile_helper.get_UUID()
        except Exception as e:
            logging.error("[_get_UUID]error is {}".format(str(e)))
        finally:
            return UUID

    def _get_oobe_date(self, path):
        """
        read the Platinum_Setting_Filename file to load the value if exists,
        otherwise it's the fisrt time to launch the OS,
        so get the current date time as the original OOBE date

        :return: OOBE date or 'unknown'
        :rtype: string
        """
        if os.path.exists(path):
            # get OOBE data from local file
            OobeDate = "unknown"
            try:
                with open(path, 'r') as input:
                    data = json.load(input)
                if data['OOBE']:
                    OobeDate = data['OOBE']
            except Exception as e:
                logging.error("[_get_oobe_date]error is {}".format(str(e)))
            finally:
                return OobeDate
        else:
            return "unknown"

    def _get_Manufacturer(self):
        """
        Get the matched BIOS item Manufacturer info

        :return: Profile manufacturer info
        :rtype: string
        """
        Manufacturer = "unknown"
        try:
            path = os.path.join('/sys/class/dmi/id', "sys_vendor")
            Manufacturer = read_sys_data(path).decode()
        except Exception as e:
            logging.error("[_get_Manufacturer]error is {}".format(str(e)))
        finally:
            return Manufacturer

    def _get_Model(self):
        """
        Get the matched BIOS item Model info

        :return: Profile model info
        :rtype: string
        """
        Model = "unknown"
        try:
            path = os.path.join('/sys/class/dmi/id', "product_name")
            Model = read_sys_data(path).decode()
        except Exception as e:
            logging.error("[_get_Model]error is {}".format(str(e)))
        finally:
            return Model

    def _get_Lob(self):
        """
        Get the matched BIOS item Lob info

        :return: Profile lob info
        :rtype: string
        """
        Lob = "unknown"
        try:
            path = os.path.join('/sys/class/dmi/id', "product_family")
            Lob = read_sys_data(path).decode()
        except Exception as e:
            logging.error("[_get_Lob]error is {}".format(str(e)))
        finally:
            return Lob

    def _get_SKU(self):
        """
        Get the matched BIOS item SKU info

        :return: Profile SKU info
        :rtype: string
        """
        SKU = "unknown"
        try:
            SKU = lsb_release.get_distro_information()['ID'] + "_client"
        except Exception as e:
            logging.error("[_get_SKU]error is {}".format(str(e)))
        finally:
            return SKU

    def _get_geoid(self):
        """
        Get the local machine's GeoId

        :return: Profile geoid info
        :rtype: string
        """
        # 1.get the linux localtion represent in Ubuntu
        # 2.read windows dictionary in local eg. {"US": 244}
        geoid = 'unknow'
        try:
            json_data = open("/usr/share/dell-linux-assistant/TZ_L2W.dat").read()
            U2W = json.loads(json_data)
            path = "/etc/timezone"
            geoid = U2W[read_sys_data(path).decode()]
        except Exception as e:
            logging.error("[_get_geoid]error is {}".format(str(e)))
        finally:
            return str(geoid)

    def _get_os_version(self):
        """
        It will return the Linux OS distribute version

        :return: Profile os version info
        :rtype: string
        """
        os_version = "unknown"
        try:
            os_version = lsb_release.get_distro_information()['RELEASE']
        except Exception as e:
            logging.error("[_get_os_version]error is {}".format(str(e)))
        finally:
            return os_version

    def _get_os_lanuage(self):
        """
        Get os language info from environment

        :return: OS language info
        :rtype: string
        """
        # get the OS environment variables
        OS_env = os.environ
        if "LANG" in OS_env.keys():
            return OS_env["LANG"]
        else:
            return None
